#include <memory>
#include <map>
#include <thread>
#include <chrono>

#include <fcf/net/socket.hpp>
#include <fcf/net/protocol/http.hpp>


#include "configuration.hpp"
#include "subprocess.hpp"

namespace application {

  class endpoint_t {
    public:
      endpoint_t(const server_configuration_t& a_server_config, size_t a_handler_index);
      ~endpoint_t();
      void run();
      void stop();
      bool is_group_exist(int a_group);
      void restart_group(int a_group);

    private:
      void _endpoint_handler(fcf::net::socket_t* a_socket);
      void _handler(fcf::net::socket_sptr_t a_socket);
      template<typename TimePoint>
      void _buffered_forwarding(const char*                             a_unsent_data_buffer,
                                size_t                                  a_unsent_data_buffer_size,
                                unsigned long long                      a_total_unset_data_size,
                                fcf::net::socket_sptr_t                 a_client_socket,
                                fcf::net::socket_sptr_t                 a_process_socket,
                                int                                     a_handler_index,
                                int&                                    a_process_index,
                                TimePoint                               a_start_response,
                                fcf::net::protocol::http_header_info_t& a_request_header_info,
                                int                                     a_max_response_timeout);

      bool                                      _slave;
      endpoint_configuration_t                  _config;
      int                                       _start_group;
      std::mutex                                _launch_mutex;
      std::thread                               _endpoint_thread4;
      std::thread                               _endpoint_thread6;
      bool                                      _stop;
      fcf::net::socket_sptr_t                   _socket4;
      fcf::net::socket_sptr_t                   _socket6;
      std::map<size_t, subprocesses_sptr_t>     _subprocesses;
      //std::vector< std::shared_ptr<handler_t> > _handlers;
  };

  typedef std::shared_ptr<endpoint_t> endpoint_sptr_t;

}
