#pragma once

#ifdef WIN32
#include <winsock2.h>
#endif
#include <memory>

namespace fcf {
namespace net {

#ifdef WIN32
  typedef SOCKET  socket_handle_t;
#else
  typedef int     socket_handle_t;
#endif

  class socket_t {
    public:
      virtual ~socket_t();
      virtual socket_t*       accept() = 0;
      virtual void            connect() = 0;
      virtual void            bind() = 0;
      virtual void            listen(int a_queueSize) = 0;
      virtual unsigned int    send(const char* a_data, unsigned int a_size) = 0;
              void            send_all(const char* a_data, unsigned int a_size);
      virtual unsigned int    recv(char* a_buffer, unsigned int a_size) = 0;
      virtual void            close() = 0;
      virtual void            set_reuse(bool a_val) = 0;
      virtual void            set_blocking(bool a_val) = 0;
      virtual void            set_timeout(int a_sec) = 0;
      virtual const char*     get_address() = 0;
      virtual int             get_port() = 0;
      virtual int             get_error() = 0;
      virtual socket_handle_t get_handle() = 0;
  };

  typedef std::shared_ptr<socket_t> socket_sptr_t;

  socket_t* create_socket(const char* a_address, int a_port);
  socket_t* create_socket_attach(socket_handle_t a_socket_handle, const char* a_address, int a_port);


} // net namespace
} // fcf namespace
